<?php
// $Id: common.inc,v 1.2.4.5 2007/11/26 04:30:37 pyutaros Exp $
/**
 * Insert new fact
 *
 * Insert new fact to Family database.
 *
 * @param $fact_code
 *   A 3-4 characters string containing a valid GEDCOM tag or a user defined tag.
 * A user defined tag must start with underscore.
 * @param $value
 *   The value to be inserted for the new fact. Value format depend on the fact.
 * @return
 *  fid (fact id) of the new fact.
 */
function family_insert_fact($fact_code, $value) {
  $fid = db_next_id('{family_facts}_fid');
  $result = db_query("INSERT INTO {family_facts} (fid, fact_code, fact_value) VALUES (%d, '%s', '%s')", $fid, $fact_code, $value);
  return ($fid);
}

function family_update_fact($fid, $value) {
  return (db_query("UPDATE {family_facts} SET fact_value = '%s' WHERE fid = %d", $value, $fid));
}

function family_remove_fact($fid) {
  //Find and remove each Subfact
  $fids = db_query("SELECT f.fid FROM {family_facts} f, {family_relations} r ".
          "WHERE f.fid=r.fid1 AND r.fid2= %d", $fid);
  while ($sub_fid = db_fetch_array($fids) ) {
    $removed = family_remove_fact($sub_fid, $fid);
  }
  //Remove both the fact & relationship to parent facts
  db_query("DELETE FROM {family_relations} WHERE fid1 = %d", $fid);
  db_query("DELETE FROM {family_facts} WHERE fid = %d", $fid);
}

function family_insert_relation($fid1, $fid2, $desc) {
  $rid = db_next_id('{family_relations}_rid');
  db_query("INSERT INTO {family_relations} (rid, fid1, fid2, relation_description) VALUES (%d, %d, %d, '%s')",
    $rid, $fid1, $fid2, $desc);
}

function family_get_subfid_by_code($fact_code, $fid) {
  $subfid = db_result(db_query("SELECT f.fid FROM {family_facts} f, {family_relations} r ".
          "WHERE (f.fact_code = '%s' AND f.fid=r.fid1 AND r.fid2= %d)", $fact_code, $fid));
  return ($subfid);
}

function family_get_subfact_by_code($fact_code, $fid) {
  $subfact = db_fetch_array(db_query("SELECT f.fid, f.nid, f.xref, f.fact_code AS code, f.fact_value AS value " .
                               "FROM {family_facts} f, {family_relations} r ".
          "WHERE (f.fact_code = '%s' AND f.fid=r.fid1 AND r.fid2= %d)", $fact_code, $fid));
  return ($subfact);
}

function family_get_subvalue_by_code($fact_code, $fid) {
  $subval = db_result(db_query("SELECT f.fact_value FROM {family_facts} f, {family_relations} r ".
          "WHERE (f.fact_code = '%s' AND f.fid=r.fid1 AND r.fid2= %d)", $fact_code, $fid));
  return ($subval);
}

function family_get_value_by_fid($fid) {
  return db_result(db_query("SELECT f.fact_value FROM {family_facts} f WHERE f.fid = '%d'", $fid));
}

function family_get_fact_by_fid($fid) {
  return db_fetch_array(db_query("SELECT fid, nid, xref, fact_code AS code, fact_value AS value "
                                ."FROM {family_facts} WHERE fid = %d", $fid));
}
function family_get_fact_by_xref($xref) {
  $xref = str_replace("@", '', $xref); //@'s are in the pointer values but not in xref database field
  return db_fetch_array(db_query("SELECT fid, nid, xref, fact_code AS code, fact_value AS value "
                                ."FROM {family_facts} WHERE xref = '%s'", $xref));
}
/*
 * This function by nature does not allow sub facts with identical fact codes.
 * With most of an individuals facts (birth, death) this works fine.
 * This breaks down if you have other facts that allow multiple of the same
 * subfacts. I would change it, but the individual include requires this returned
 * data structure and I don't want to rewrite it right now. It is badly named anyway.
 * So,  depreciated in favor of a better named function returning a slightly
 * revised data structure: family_get_subfacts
 */
function family_getfacts($fid)
{
  $personfact = array();
  $person = db_query("SELECT {family_facts}.fid, {family_facts}.fact_code, {family_facts}.fact_value "
          .          "FROM {family_facts} "
          .          "INNER JOIN {family_relations} ON {family_facts}.fid = {family_relations}.fid1 "
          .          "AND {family_relations}.fid2 = %d ", $fid);
  while($attribs = db_fetch_array($person)) {
    $personfact[$attribs['fact_code']]=array('value' => $attribs['fact_value'], 'fid' => $attribs['fid']);
  }
  return $personfact;
}
/*
 * The multidimensional arrays allow us to quickly retrieve the desired code by name without clobering
 * duplicates at the cost of preserving order among sub facts.
 * The only special case requiring order is CONT & CONC.  (see family_get_cont)
 * I left value as an associative array to allow attaching further subfacts to the array
 */
function family_get_subfacts($fid)
{
  $subfacts = array();
  $results = db_query("SELECT ff.fid, ff.nid, ff.xref, ff.fact_code as code, ff.fact_value as value "
          .          "FROM {family_facts} ff "
          .          "INNER JOIN {family_relations} ON ff.fid = {family_relations}.fid1 "
          .          "AND {family_relations}.fid2 = %d ", $fid);
  while($attribs = db_fetch_array($results)) {
    $subfacts[$attribs['code']][]=array('value' => $attribs['value'],
                                             'fid'   => $attribs['fid'],
                                             'xref'  => $attribs['xref'],
                                             'nid'   => $attribs['nid']);
  }
  return $subfacts;
}

function family_get_parent_fact($fid)
{
  return db_fetch_array(
           db_query(
             "SELECT ff.fid, ff.nid, ff.xref, ff.fact_code AS code, ff.fact_value AS value "
          .  "FROM {family_facts} ff "
          .  "INNER JOIN {family_relations} ON ff.fid = {family_relations}.fid2 "
          .  "AND {family_relations}.fid1 = %d ", $fid));
}

/**
 * Find the individual node owning a given fact (fid), i.e. root of a tree
 * Return an array of fact values
 */
function family_get_fact_root($fid) {
  $curr_fid = $fid;
  $end = false;
  do {
      $parent = db_result(db_query("SELECT fid2 ".
                                   "FROM {family_relations} " .
                                   "WHERE relation_description = 'FACT'".
                                   "AND fid1 = %d",$curr_fid));
      if($parent) {
        $curr_fid = $parent;
      }
      else {
        $end = true;
      }

  } while (!$end);

  return db_fetch_array(db_query("SELECT fid, nid, fact_code, fact_value " .
                             "FROM {family_facts} " .
                             "WHERE fid = %d", $curr_fid));
}

/**
 * Check if record $fid is private.
 */
function family_check_privacy($fid) {
  if ($privacy_fid = family_get_subfid_by_code("_PRV", $fid)){
    $privacy = family_get_subvalue_by_code("_PRV", $fid);
  }
  else {
    $privacy=0;
  }
  if (($privacy==0) || ($privacy==1) || ($privacy==3)) {
    return user_access('access family');
  } else {
    return user_access('access family private records');
  }
}

/**
 * Retrieves and combines CONT & CONC fact texts
 */
function family_get_cont($fid){
  $text="";
  $results = db_query("SELECT {family_facts}.fact_code, {family_facts}.fact_value "
          .          "FROM {family_facts} "
          .          "INNER JOIN {family_relations} ON {family_facts}.fid = {family_relations}.fid1 "
          .          "AND {family_relations}.fid2 = %d "
          .          "WHERE {family_facts}.fact_code LIKE 'CONT' "
          .          "OR    {family_facts}.fact_code LIKE 'CONC' "
          .          "ORDER BY {family_facts}.fid ASC", $fid); //Order required for CONT & CONC
  while($attribs = db_fetch_array($results)) {
    if (strcmp($attribs['fact_code'], "CONT") == 0 ) {$text.="<br />";}
  	$text .= $attribs['fact_value'];
  }
  return $text;
}

/**
 * Returns the family members (INDI fact) for a family group
 * Optionally provide a list of desired famnily member types ('HUSB', 'WIFE', 'CHIL')
 * E.g., if you only want the couple: array('HUSB','WIFE')
 * Members are returned not ordered (neither fids nor xrefs are suitable for sorting)
 */
function family_get_fam_by_famid($famid, $type_arr=array('HUSB', 'WIFE', 'CHIL')) {
 	$type_text = "'".implode("', '", $type_arr)."'";
 	$family = array();
 	$results = db_query("SELECT ff.fid, ff.nid, ff.fact_code as code, ff.xref, " .
 			           "  ff.fact_value as value, fr.relation_description " .
 	                   "FROM {family_facts} ff, {family_relations} fr " .
 	                   "WHERE fr.fid2 = %d " .
 	                   "AND fr.fid1 = ff.fid " .
                     // I know this isn't the most secure way to create the IN values,
                     // but a prepared statment will escape the single quotes.
                     "AND fr.relation_description IN ($type_text) ", $famid);

    while($indi = db_fetch_array($results)){
    	$family[$indi['relation_description']][]=array(
    	  'fid'   => $indi['fid'  ],
    	  'nid'   => $indi['nid'  ],
    	  'code'  => $indi['code' ],
    	  'xref'  => $indi['xref' ],
          'value' => $indi['value'],
    	);
    }
    return $family;
}

function family_get_address($address_fact) {
  $content .= "<div>Address:<br />".$address_fact['value'];

  $addr_subfacts = family_get_subfacts($address_fact['fid']);
  if($addr_subfacts['CONT']) {// A very long address line ...
      $content .= family_get_cont($address_fact['fid']);
  }
  if($addr_subfacts['ADR1']) {
      $content .= '<br />' . $addr_subfacts['ADR1'][0]['value'];
  }
  if($addr_subfacts['ADR2']) {
    $content .= '<br />' . $addr_subfacts['ADR2'][0]['value'];
  }
  if($addr_subfacts['ADR3']) {
    $content .= '<br />' . $addr_subfacts['ADR3'][0]['value'];
  }
  if ($addr_subfacts['CITY']) {
    $content .= '<br />'.$addr_subfacts['CITY'][0]['value'];
  }
  // Check for proper separator
  if ($addr_subfacts['CITY'] && $addr_subfacts['STAE']) {
  	$content .= ", " . $addr_subfacts['STAE'][0]['value'];
  }
  else {
  	$content .= '<br />'.$addr_subfacts['STAE'][0]['value'];
  }
  // Check for proper separator
  if ($addr_subfacts['POST'] && $addr_subfacts['STAE']) {
  	$content .= ' ' . $addr_subfacts['POST'][0]['value'];
  }
  else {
  	$content .= '<br />' . $addr_subfacts['POST'][0]['value'];
  }
  if ($addr_subfacts['CTRY']) {
  	$content .= '<br />'.$addr_subfacts['CTRY'][0]['value'];
  }

  return $content . "</div>";
}