<?php
// $Id: import.inc,v 1.5.4.5 2007/11/26 04:30:37 pyutaros Exp $
require_once "includes/common.inc";
// import.inc
// Functions for importing GEDCOM files to database
// Using the data base defined in simple.mysql
// This may also be used as a temporary storage before making more processing
// for import to other database format.

ini_set('auto_detect_line_endings', true);

//Generate a form for uploading a GEDCOM file
function family_import() {
  return drupal_get_form('family_import_form');
}

function family_import_form() {
  $form['#attributes'] = array('enctype' => "multipart/form-data");
  $form['gedcom_file'] = array(
    '#type' => 'file',
    '#title' => t('GED file to upload'),
    '#size' => 40,
  );
  //$form['merge'] = array(
  //  '#type' => 'radios',
  //  '#title' => t('Merge options'),
  //  '#options' => array(t('replace existing data'), t('augment current data'), t('merge individuals by name')),
  //  '#default_value' => variable_get('family_import_replace', 1),
  //);

  $form['range'] = array(
    '#type' => 'fieldset',
    '#title' => t('Import range'),
    '#description' => t('Select a range of records (lines staring with 0) to import.  This allows breaking very large files into multiple import sessions.'),
  );
  $form['range']['start'] = array(
    '#type' => 'textfield',
    '#title' => t('First record to import'),
    '#size' => 10,
    '#maxlength' => 10,
    '#description' => t('Enter the number of the first record in the GEDCOM file to include in this import session'),
  );
  $form['range']['nrecords'] = array(
    '#type' => 'textfield',
    '#title' => t('Number of records to import'),
    '#size' => 10,
    '#maxlength' => 10,
    '#description' => t('Enter the number of records to process in this import session'),
  );
  $form['replace'] = array(
    '#type' => 'checkbox',
    '#title' => t('Replace existing GED data'),
    '#default_value' => variable_get('family_import_replace', 1),
  );
  $form['submit'] = array('#type' => 'submit', '#value' => t('Start Import'));
  return $form;
}

// Check the uploaded GEDCOM file
function family_import_form_validate($form_id, $form_values) {
  $file = file_check_upload('gedcom_file');
  if (!$file) {
    form_set_error('',t("Didn't get GED file"));
  }
  
  $fp = fopen($file->filepath , "r" );
  if (!$fp) {
    form_set_error('',t("Couldn't open get GED file"));
  }
  fclose($fp);
}

// Parse the uploaded GEDCOM file
function family_import_form_submit($form_id, $form_values) {
  $file = file_check_upload('gedcom_file');
  if (!$file) {
    form_set_error('',t("Didn't get GED file"));
  }
  
  $fp = fopen($file->filepath , "r" );
  if (!$fp) {
    form_set_error('',t("Couldn't open get GED file"));
  }

  
  //
  // Empty current content. This is useful for debugging, but more caution should be
  // done before deleting database in the working version
  //
  if ($form_values['replace'])
  {
    db_query("TRUNCATE {family_facts}");
    db_query("TRUNCATE {family_relations}");
  
    $q = db_query("select nid from {node} where type = 'family_individual'");
    $n = 0;
    while ($o = db_fetch_object($q)) {
      node_delete($o->nid);
      $n++;
    }
    drupal_set_message(t('Deleted @n family_individual nodes.', array('@n' => $n)));
  }

  $rmin = $form_values['start']? $form_values['start']:0;
  $rcount = $form_values['nrecords']? $form_values['nrecords']:99999999;
  $rmax = $rmin + $rcount - 1;
  $rnum = 0;
  $rprocessed = 0;
  $lprocessed = 0;
  
  $lnum = 0;
  $gedcom_hier=array();   // References to GEDCOM parents on each level
  
  while (!feof ($fp))
  {
    $gedline = fgets( $fp, 1024 );
    $lnum++;
    
    if (preg_match("/^\s*(\d+)\s*(?:@([^@]+)@)?\s*(\S+)\s*(.*\S)?\s*$/i", $gedline , $matches))
    {
      $level=$matches[1];
      if ($level == 0) ++$rnum;
      if ($rnum < $rmin) continue;
      if ($rnum > $rmax) break;
      if ($level == 0) ++$rprocessed;
      ++$lprocessed;
      $xref=$matches[2];
      $fact_code=$matches[3];
      $value=$matches[4];
      $gedcom_source=$gedline;
      $parent=$gedcom_hier[$level-1];

      if (strpos(";HUSB;WIFE;CHIL",$fact_code))
      {
        //
        // Lines that define only a relation
        //
        $gedcom_hier[$level]=NULL;
        preg_match("/@\s*([^@\s]+)\s*@/i", $gedline , $matches);
        $target_xref=$matches[1];
        $fid = db_result(db_query("SELECT fid FROM {family_facts} WHERE xref = '%s'", $target_xref));
        $relation=$fact_code;
      }
      else
      {
        //
        // Lines that define a fact
	// Every INDI fact gets a node
	//
        $fid = db_next_id('{family_facts}_fid');
        $nid=NULL;
        $relation="FACT";

        if ($fact_code == 'INDI') {
          unset($node);
          $node->type = "family_individual";
          $node->uid = $user->uid;
          $node->title = "Unknown";
          $node->status = 1;
          $node->moderate = 0;
          $node->comment = 2;
          $node->revision = 0;
          $node->fid = $fid;
          $node->xref = $xref;
          $node->value = $value;
          $node->gedcom_source = $gedcom_source;
          node_validate($node, $error);
          if (!node_access("create", $node)) {
            $error['access'] = message_access();
          }
          if ($error) {
   	    drupal_set_message(
              t(
		'Error at line @lnum of GED (@line): @error.',
              	array('@lnum' => $lnum, '@line' => $gedline, '@error' => print_r($error,true))
              )
	    );
          }
          else {
            node_save($node);
            $nid=$node->nid;
          }
          unset($node);
        }
	else
	{
          db_query("INSERT INTO {family_facts} (fid, nid, xref, fact_code, fact_value, gedcom_source)
	            VALUES (%d, %d, '%s', '%s', '%s', '%s')",
	  	    $fid, $nid, $xref,$fact_code,$value,$gedcom_source);
	}
        $gedcom_hier[$level] = $fid;
        $gedcom_source=null; // If the source is in fact table, no need to put it again in relation table
    	$nchanged++;
      }

      if ($level>0) {
        $rid = db_next_id('{family_relations}_fid');
        db_query("INSERT INTO {family_relations} (rid, fid1, fid2, relation_description,
                       gedcom_source) VALUES (%d, %d, %d, '%s', '%s')",
                       $rid,$fid, $parent, $relation, $gedcom_source);
      }
    }
  }
  fclose ($fp);
  
  drupal_set_message(t('Processed @r records (@n lines) of GED.', array('@r' => $rprocessed, '@n' => $lprocessed)));
  if ($rnum > $rmax) drupal_set_message(t('Next start record: @r.', array('@r' => $rmax + 1)));
  else drupal_set_message(t('No more records to process'));

  //
  // Clean up: add appropriate Titles to nodes based on the NAME fact
  // -- An alternative to this is to cache created nodes until we
  //    have added all the family_facts.  That has the benefit of
  //    being easy to roll back if there are errors in the GED file
  //    but it uses more ram for a big import.  I may try this if
  //    my big GED still takes too long to import... --pf
  //
  $fids = db_query("SELECT f1.fid as fid, f1.nid as nid, f2.fact_value as name ".
  	"FROM {family_facts} f1, {family_facts} f2, {family_relations} r ".
	"WHERE f1.fact_code = 'INDI' AND f2.fact_code='NAME' AND f2.fid=r.fid1 AND r.fid2=f1.fid");
  while ($row = db_fetch_array($fids)) {
    $name=str_replace("/","",$row['name']); //Remove slashes arround surname

    //UPDATE NAME
    $node=node_load($row['nid']);
    if ($node->title != $name)
    {
      $node->title = $name;
      node_save($node);
    }
  }
  return 'family';
}

