<?php
// $Id: individual.inc,v 1.2.4.5 2007/11/26 04:30:37 pyutaros Exp $
/**
 * Implementation of hook_access().
 */
function family_individual_access($op, $node) {
  if ($op == 'create' || $op == 'update' || $op == 'delete') {
    return user_access('add/edit/delete family');
  }
  if ($op == 'view') {
    return family_check_privacy($node->fid);
  }
}

/**
 * Implementation of hook_form().
 */
function family_individual_form(&$node) {
  $form['title'] = array(
    '#type' => 'hidden',
    '#default_value' => $node->NAME,
  );
  $form['GIVN'] = array(
    '#type' => 'textfield',
    '#title' => t('Given Name'),
    '#default_value' => $node->GIVN,
    '#required' => FALSE,
  );
  $form['SURN'] = array(
    '#type' => 'textfield',
    '#title' => t('Surname'),
    '#default_value' => $node->SURN,
    '#required' => FALSE,
  );
  $form['SEX'] = array(
    '#type' => 'select',
    '#title' => t('Gender'),
    '#default_value' => $node->SEX,
    '#options' => array(
      '?' => 'Unknown',
      'M' => 'Male',
      'F' => 'Female'),
  );
  $form['BIRT']['BIRT_DATE'] = array(
    '#type' => 'textfield',
    '#title' => t('Birth Date'),
    '#default_value' => $node->BIRT_DATE,
    '#required' => FALSE,
  );
  $form['BIRT']['BIRT_PLAC'] = array(
    '#type' => 'textfield',
    '#title' => t('Place'),
    '#default_value' => $node->BIRT_PLAC,
  );
  $form['DEAT']['DEAT_DATE'] = array(
    '#type' => 'textfield',
    '#title' => t('Death Date'),
    '#default_value' => $node->DEAT_DATE,
  );
  $form['DEAT']['DEAT_PLAC'] = array(
    '#type' => 'textfield',
    '#title' => t('Place'),
    '#default_value' => $node->DEAT_PLAC,
  );
  $form['body_filter']['body'] = array(
    '#type' => 'textarea',
    '#title' => t('Biography'),
    '#default_value' => $node->body,
    '#rows' => 20,
    '#required' => FALSE
  );
  $form['body_filter']['format'] = filter_form($node->format); //Not sure why this goes here, but all the examples have it...
  return $form;
} // function family_individual_form(&$node, &$param)

/**
 * Implementation of hook_insert().
 *  Insert All the facts and relationships
 *  Update Node and revision to replace data not in form
 *
 *  RELATIONSHIPS:  Base facts are related to INDI (fact, INDI)
 *                  Subfacts are related to super fact  (sub, super)
 *  NOTE:
 *  Should we generate the gedcom code at the same time,
 *  or leave that for exporting?
 */
function family_individual_insert($node) {
  //Insert Individual Line - TODO: Need to be able to assign a new xref & insert Gedcom code
  if ($node->fid) {
    $ind_fid=$node->fid;
  }
  else {
    $ind_fid = db_next_id('{family_facts}_fid');
  }
  db_query("INSERT INTO {family_facts} (fid, nid, xref, fact_code, fact_value,
                         gedcom_source) VALUES (%d, %d, '%s', '%s', '%s', '%s')", $ind_fid,
                        $node->nid, $node->xref,'INDI',$node->value,$node->gedcom_source);

  //INSERT NAME
  if (($node->SURN or $node->GIVN)) {
    $node->NAME = $node->GIVN . ' /' . $node->SURN . '/';
    $node->title = $node->GIVN . ' ' . $node->SURN;
  }
  if ($node->NAME) {
    $name_fid = family_insert_fact('NAME', $name);
    family_insert_relation($name_fid, $ind_fid, 'FACT');
  }

  //INSERT SEX
  if ($node->SEX) {
    $fid =  family_insert_fact('SEX', $node->SEX);
    family_insert_relation($fid, $ind_fid, 'FACT');
  }

  //INSERT BIRT if either date or place exist (At least a birth date will appear, since it is required in the form)
  if ($node->BIRT_DATE or $node->BIRT_PLAC){
    $birt_fid = family_insert_fact('BIRT', '');
    family_insert_relation($birt_fid, $ind_fid,'FACT');
    //INSERT DATE & RELATION
    if ($node->BIRT_DATE){
      $fid = family_insert_fact('DATE', $node->BIRT_DATE);
      family_insert_relation($fid, $birt_fid, 'FACT');
    }
    if ($node->BIRT_PLAC){
      //INSERT PLAC & RELATION
      $fid = family_insert_fact('PLAC', $node->BIRT_PLAC);
      family_insert_relation($fid, $birt_fid, 'FACT');
    }
  }
  //INSERT DEAT if either date or place exist
  if ($node->DEAT_DATE or $node->DEAT_PLAC){
    $deat_fid = family_insert_fact('DEAT', '');
    family_insert_relation($deat_fid, $ind_fid,'FACT');
    //INSERT DATE
    if ($node->DEAT_DATE){
      $fid = family_insert_fact('DATE', $node->DEAT_DATE);
      family_insert_relation($fid, $deat_fid, 'FACT');
    }
    //INSERT PLAC
    if ($node->DEAT_PLAC){
      $fid = family_insert_fact('PLAC', $node->DEAT_PLAC);
      family_insert_relation($fid, $deat_fid, 'FACT');
    }
  }
}

/**
 * Implementation of family_individual_load().
 * This function will grab all the descriptive data for an individual
 * Primary use is for filling in form data to preform an edit
 * [Code mostly taken from family_line_indi()]
 *
 * Returns an array of additions
 */
function family_individual_load($node) {
  $nid = $node->nid;
  //If they don't exist then there is no point in going on so return the empty array
  $additions->fid = db_result(db_query("SELECT fid FROM {family_facts} WHERE nid = %d",$nid));
  //NAME
  if ($name = family_get_subvalue_by_code("NAME", $additions->fid)){
    if (preg_match("/^(.*)\/(.*)\/(.*)$/i", $name , $matches)) {
      $additions->SURN = $matches[2];
      $additions->GIVN = $matches[1].$matches[3];
    }
    $additions->NAME = str_replace("/","",$name); //Remove slashes arround surname
  }
  //Gender
  $gender = family_get_subvalue_by_code("SEX",$additions->fid);
  if ($gender) {$additions->SEX = $gender;}
  //BIRTH
  if ($birt_fid = family_get_subfid_by_code("BIRT", $additions->fid)) {
    $birt_date = family_get_subvalue_by_code("DATE", $birt_fid);
    if ($birt_date) {
      $additions->BIRT_DATE = $birt_date;
    }
    $birt_plac = family_get_subvalue_by_code("PLAC", $birt_fid);
    if ($birt_plac) {
      $additions->BIRT_PLAC = $birt_plac;
    }
  }
  //DEATH
  if ($deat_fid = family_get_subfid_by_code("DEAT", $additions->fid)) {
    $deat_date = family_get_subvalue_by_code("DATE", $deat_fid);
    if ($deat_date) {
      $additions->DEAT_DATE = $deat_date;
    }
    $deat_plac = family_get_subvalue_by_code("PLAC", $deat_fid);
    if ($deat_plac){
      $additions->DEAT_PLAC = $deat_plac;
    }
  }
  return($additions);
}

/**
 * Implementation of hook_update().
 *  Update the facts and new relationships
 *  Update Node and revision to replace data not in form
 *
 *  See family_individual_insert
 */
function family_individual_update($node) {
  //UPDATE NAME
  if (!$node->GIVN) {
    $node->GIVN = "Unknown";
  }
  if (!$node->SURN) {
    $node->SURN = "Unknown";
  }
  $node->NAME = $node->GIVN . ' /' . $node->SURN . '/';
  if ($name_fid = family_get_subfid_by_code("NAME", $node->fid)) {
    family_update_fact($name_fid, $node->NAME);
  }
  else {
    $name_fid = family_insert_fact('NAME', $name);
    family_insert_relation($name_fid, $node->fid, 'FACT');
  }
  //Create a node title using full name
  $node->title = str_replace("/","",$node->NAME);
  //UPDATE GENDER
  drupal_set_message("gender '".$node->SEX."'");
  if (($node->SEX=='F') or ($node->SEX=='M')) {
    if ($gender_fid = family_get_subfid_by_code("SEX", $node->fid)) {
      family_update_fact($gender_fid, $node->SEX);
    }
    else{
      $fid = family_insert_fact('SEX', $node->SEX);
      family_insert_relation($fid, $node->fid, 'FACT');
    }
  }
  elseif ($gender_fid = family_get_subfid_by_code("SEX", $node->fid)) {
    family_remove_fact($gender_fid);
  }
  //UPDATE BIRT
  //IF there is birth data, we need to get it to update it
  if ($birt_fid = family_get_subfid_by_code('BIRT', $node->fid)) {
    $birt_date_fid = family_get_subfid_by_code('DATE', $birt_fid);
    $birt_plac_fid = family_get_subfid_by_code('PLAC', $birt_fid);
  }
  //IF either BIRTH parts are listed, we need to make sure we have a BIRT
  if (($node->BIRT_DATE == '') && ($node->BIRT_PLAC == '')) {
    family_remove_fact($birt_fid);
  }
  elseif (! $birt_fid) {
    $birt_fid = family_insert_fact('BIRT', '');
    family_insert_relation($birt_fid, $node->fid,'FACT');
  }
  //UPDATE BIRT DATE & RELATION
  //IF the node has a date OR a date exists
  if ($birt_date_fid && ($node->BIRT_DATE == '') ){ family_remove_fact($birt_date_fid); }
  elseif ($birt_date_fid) {
    family_update_fact($birt_date_fid, $node->BIRT_DATE);
  }
  elseif ($node->BIRT_DATE != '') {
    $fid = family_insert_fact('DATE', $node->BIRT_DATE);
    family_insert_relation($fid, $birt_fid, 'FACT');
  }
  //UPDATE BIRT PLAC & RELATION
  if ($birt_plac_fid && ($node->BIRT_PLAC == '') ){ family_remove_fact($birt_plac_fid); }
  elseif ($birt_plac_fid) {
    family_update_fact($birt_plac_fid, $node->BIRT_PLAC);
  }
  elseif ($node->BIRT_PLAC != '') {
    $fid = family_insert_fact('PLAC', $node->BIRT_PLAC);
    family_insert_relation($fid, $birt_fid, 'FACT');
  }
  //UPDATE DEAT
  $deat_fid = family_get_subfid_by_code('DEAT', $node->fid);
  if ($deat_fid) {
    $deat_date_fid = family_get_subfid_by_code('DATE', $deat_fid);
    $deat_plac_fid = family_get_subfid_by_code('PLAC', $deat_fid);
  }
  if (($node->DEAT_DATE == '') && ($node->DEAT_PLAC == '')) {
    family_remove_fact($deat_fid);
  }
  elseif (!$deat_fid) {
    $deat_fid = family_insert_fact('DEAT', '');
    family_insert_relation($deat_fid, $node->fid,'FACT');
  }
  //UPDATE DEAT DATE & RELATION
  if ($deat_date_fid && ($node->DEAT_DATE == '')) {
    family_remove_fact($deat_date_fid);}
  elseif ($deat_date_fid) {
    family_update_fact($deat_date_fid, $node->DEAT_DATE);
  }
  elseif ($node->DEAT_DATE != '') {
    $fid = family_insert_fact('DATE', $node->DEAT_DATE);
    family_insert_relation($fid, $deat_fid, 'FACT');
  }
  //UPDATE DEAT PLAC & RELATION
  if ($deat_plac_fid && ($node->DEAT_PLAC == '')) {
    family_remove_fact($deat_plac_fid);
  }
  elseif ($deat_plac_fid) {
    family_update_fact($deat_plac_fid, $node->DEAT_PLAC);
  }
  elseif ($node->DEAT_PLAC != '') {
    $deat_plac_fid = family_insert_fact('PLAC', $node->DEAT_PLAC);
    family_insert_relation($deat_plac_fid, $deat_fid, 'FACT');
  }
}

/**
 * Implementation of hook_view().
 */
function family_individual_view(&$node, $teaser = FALSE, $page = FALSE) {
  $node = node_prepare($node, $teaser);
//  $fid = db_result(db_query("SELECT fid FROM {family_facts} WHERE nid= %d", $node->nid));
//  $node->body = family_view_indi($node->fid) . $node->body;
//  $node->teaser = family_view_indi($node->fid) . $node->teaser;
  $node->content['family_individual_view'] = array(
      '#value' => theme($teaser? 'family_individual_teaser':'family_individual_body', $node),
      '#weight' => 0,
  );

  return $node;
}

function theme_family_individual_teaser($node) {
  return family_view_indi($node->fid);
}
function theme_family_individual_body($node) {
  return family_view_indi($node->fid);
}
/**
 * Implementation of hook_delete().
 */
function family_individual_delete(&$node) {
//  $ind_fid = db_result(db_query("SELECT fid FROM {family_facts} WHERE nid = %d",$node->nid));
  family_remove_fact($node->fid);
}

/**
 * Just so we don't have to keep formatting a name over and over
 */
function family_get_display_name($name){
    $name_arr = array('SURN'=>'', 'GIVN'=>'', 'NAME'=>'');
    if (preg_match("/^(.*)\/(.*)\/(.*)$/i", $name , $matches)) {
      $name_arr['SURN'] = strtoupper($matches[2]);
      $name_arr['GIVN'  ] = $matches[1].$matches[3];
      if (empty($name_arr['SURN'])) {
        $name_arr['SURN'] = "Unknown";
      }
      if (empty($name_arr['GIVN'])) {
        $name_arr['GIVN'] = "Unknown";
      }
    }
    $name_arr['NAME']=$name_arr['GIVN'].' '.$name_arr['SURN'];
    return $name_arr;
}